﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using CreateWordSample.Models;
using Syncfusion.DocIO.DLS;
using System.IO;
using Syncfusion.DocIO;
using Syncfusion.DocIORenderer;
using Microsoft.AspNetCore.Hosting;
using Syncfusion.Pdf;

namespace CreateWordSample.Controllers
{
    public class HomeController : Controller
    {
        private IHostingEnvironment _env;
        private Dictionary<Object, string> fields = null;
        public HomeController(IHostingEnvironment env)
        {
            _env = env;
        }

        public IActionResult Index()
        {
            return View();
        }
        public ActionResult CreateDocument()
        {
            FileStream fileStream = new FileStream(System.IO.Path.Combine(_env.WebRootPath, "Template.docx"), FileMode.Open);
            WordDocument document = new WordDocument(fileStream, FormatType.Docx);
            document.UpdateDocumentFields();
            fields = RetriveFieldsFromDocument(document);            
            //Saves the document
            FileStream outputStream = new FileStream(_env.WebRootPath + @"\Syncfusion_Output.docx", FileMode.Create);
            document.Save(outputStream, FormatType.Docx);
            //Closes the instance of document object
            document.Close();
            outputStream.Position = 0;          
            //Download Word document in the browser
            return File(outputStream, "application/msword");
        }

        private Dictionary<object, string> RetriveFieldsFromDocument(WordDocument document)
        {
            fields = new Dictionary<object, string>();
            //Iterates each section and get the tables.
            foreach (WSection section in document.Sections)
            {
                RetriveFieldsInTextBody(section.Body);
            }
            return fields;
        }

        private void RetriveFieldsInTextBody(WTextBody textBody)
        {
            for (int i = 0; i < textBody.ChildEntities.Count; i++)
            {
                //IEntity is the basic unit in DocIO DOM.                 
                IEntity bodyItemEntity = textBody.ChildEntities[i];

                //A Text body has 3 types of elements - Paragraph, Table, and Block Content Control
                //Decides the element type by using EntityType
                switch (bodyItemEntity.EntityType)
                {
                    case EntityType.Paragraph:
                        WParagraph paragraph = bodyItemEntity as WParagraph;
                        RetriveFieldsFromParagrap(paragraph.Items);
                        break;

                    case EntityType.Table:
                        //Table is a collection of rows and cells
                        //Iterates through table's DOM                        
                        RetriveFieldsFromTable(bodyItemEntity as WTable);
                        break;

                    case EntityType.BlockContentControl:
                        BlockContentControl blockContentControl = bodyItemEntity as BlockContentControl;
                        RetriveFieldsInTextBody(blockContentControl.TextBody);
                        break;
                }
            }
        }

        private void RetriveFieldsFromTable(WTable table)
        {
            //Iterates the row collection in a table
            foreach (WTableRow row in table.Rows)
            {
                //Iterates the cell collection in a table row
                foreach (WTableCell cell in row.Cells)
                {
                    RetriveFieldsInTextBody(cell);
                }
            }
        }

        private void RetriveFieldsFromParagrap(ParagraphItemCollection paraItems)
        {
            for (int i = 0; i < paraItems.Count; i++)
            {
                if (paraItems[i] is WField)
                {
                    fields.Add(paraItems[i], (paraItems[i] as WField).FieldValue);
                    // if you want to get field result, then get field.Text from the field.
                }               
            }
        }

        public IActionResult About()
        {
            ViewData["Message"] = "Your application description page.";

            return View();
        }

        public IActionResult Contact()
        {
            ViewData["Message"] = "Your contact page.";

            return View();
        }

        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}
